#ifndef ATM168_H_
#define ATM168_H_

//Various useful defines
#define ON 1
#define OFF 0

//Hardware IO defs
typedef union port_b
{
	unsigned char value;
	struct 
	{
		unsigned char icp1:1;				//SONAR ICP pin
		unsigned char pan:1;				//sensor turret pan
		unsigned char tilt:1;				//sensor turret tilt
		unsigned char leftDir:1;			//left motor direction
		unsigned char rightDir:1;			//right motor direction
		unsigned char fireSense:1;			//fire detector
		unsigned char none:2;				//OSC1&2, not used
	};
	struct 
	{
		unsigned char b0:1;
		unsigned char b1:1;
		unsigned char b2:1;
		unsigned char b3:1;
		unsigned char b4:1;
		unsigned char b5:1;
		unsigned char b6:1;
		unsigned char b7:1;
	};
} port_b;
#define PORTBbits (*((volatile port_b *)(&PORTB)))
#define PINBbits (*((volatile port_b *)(&PINB)))

typedef union port_c
{
	unsigned char value;						//whole 8 bits
	struct
	{
		unsigned char edges:4;					//first four bits are edge detect/line follower
		unsigned char flame1:1;
		unsigned char flame2:1;
		unsigned char none:2;					//unused bits
	};
	struct
	{
		unsigned char b0:1;						//Generic bits
		unsigned char b1:1;
		unsigned char b2:1;
		unsigned char b3:1;
		unsigned char b4:1;
		unsigned char b5:1;
		unsigned char b6:1;
		unsigned char b7:1;		
	};
} port_c;
#define PORTCbits (*((volatile port_c *)(&PORTC)))
#define PINCbits (*((volatile port_c *)(&PINC)))

typedef union port_d
{
	unsigned char value;						//whole 8 bits
	struct
	{
		unsigned char RxD:1;				//USART
		unsigned char TxD:1;				//USART
		unsigned char leftInt:1;			//Left motor odometry interrupt
		unsigned char rightInt:1;			//right motor odometry interrupt
		unsigned char motEn:1;				//motor driver enable
		unsigned char leftPWM:1;			//motor PWM
		unsigned char rightPWM:1;			//motor PWM
		unsigned char trigger;				//SONAR trigger
	};
	struct 
	{
		unsigned char b0:1;						//Generic bits
		unsigned char b1:1;
		unsigned char b2:1;
		unsigned char b3:1;
		unsigned char b4:1;
		unsigned char b5:1;
		unsigned char b6:1;
		unsigned char b7:1;		
	};
} port_d;
#define PORTDbits (*((volatile port_d *)(&PORTD)))// Defines that are, for some reason, not in the correct pin location

//Configurations for hardware devices (timers/PWM/counters/etc.)

#define SPIN			PORTDbits.b7	//servo
#define YELLOW_LED		PORTBbits.b4	//Yellow LED on the bootloader board
#define STATUS_LED		PORTBbits.b1	//LED on PB1
#define OTHER_LED		PORTBbits.b2	//LED next to PB1, on PB2

void initialize();
void waitms(unsigned int d);

#endif /*ATM168_H_*/
